@@ -4,7 +4,7 @@
 
 
 package Time::Moment::Epoch;
-our $VERSION = '1.000'; # VERSION
+our $VERSION = '1.002'; # VERSION
 
 use v5.10;
 use strict;
@@ -17,40 +17,6 @@ use Time::Moment;
 my $SECONDS_PER_DAY = 24 * 60 * 60;
 my $NANOSECONDS_PER_DAY = $SECONDS_PER_DAY * 1e9;
 
-=head1 NAME
-
-Time::Moment::Epoch
-
-=head1 DESCRIPTION
-
-Convert various epoch times to and from datetimes using L<Time::Moment>.
-
-=head1 SYNOPSIS
-
-	use Time::Moment::Epoch;
-
-	say Time::Moment::Epoch::unix(1234567890);
-			# 2009-02-13T23:31:30Z
-	say Time::Moment::Epoch::to_unix('2009-02-13T23:31:30Z');
-			# 1234567890
-
-	say Time::Moment::Epoch::chrome(12879041490654321);
-			# 2009-02-13T23:31:30.654321Z
-	say Time::Moment::Epoch::to_chrome('2009-02-13T23:31:30.654321Z');
-			# 12879041490654321
-
-=head1 CONVERSIONS
-
-The following functions convert an epoch of the specified type to a Time::Moment object.
-
-They each have a corresponding C<to_$type> function which accepts a datetime string (in any format accepted by the C<from_string> method of L<Time::Moment>) and returns the corresponding epoch.
-
-=head2 apfs
-
-APFS time is the number of nanoseconds since the Unix epoch. Cf., APFS
-filesystem format (https://blog.cugu.eu/post/apfs/).
-
-=cut
 
 # APFS time is in nanoseconds since the Unix epoch.
 sub apfs {
@@ -63,11 +29,6 @@ sub to_apfs {
 }
 
 
-=head2 chrome
-
-Chrome time is the number of microseconds since S<1601-01-01>.
-
-=cut
 
 # Chrome time is the number of microseconds since 1601-01-01, which is
 # 11,644,473,600 seconds before the Unix epoch.
@@ -81,11 +42,6 @@ sub to_chrome {
 	_time2epoch($tm, 1_000_000, -11_644_473_600);
 }
 
-=head2 cocoa
-
-Cocoa time is the number of seconds since S<2001-01-01>.
-
-=cut
 
 # Cocoa time is the number of seconds since 2001-01-01, which
 # is 978,307,200 seconds after the Unix epoch.
@@ -98,11 +54,55 @@ sub to_cocoa {
 	_time2epoch($tm, 1, 978_307_200);
 }
 
-=head2 google_calendar
 
-Google Calendar time is 32-day months from the day before the Unix epoch.
+# DOS time uses bit fields to store dates between 1980-01-01 and
+# 2107-12-31 (it fails outside that range).
+sub dos {
+	my $num = shift;
+
+	my $year   = ($num >> 25) & 0b1111111;
+
+	my $month  = ($num >> 21) &    0b1111;
+	return if $month < 1 or $month > 12;
+
+	my $day    = ($num >> 16) &   0b11111;
+	return if $day < 1 or $day > 31;
+
+	my $hour   = ($num >> 11) &   0b11111;
+	return if $hour < 0 or $hour > 23;
+
+	my $minute = ($num >>  5) &  0b111111;
+	return if $minute < 0 or $minute > 60;
+
+	my $second = ($num      ) &   0b11111;
+	return if $second < 0 or $second > 60;
+
+	Time::Moment->new(
+		year   => 1980 + $year,
+		month  => $month,
+		day    => $day,
+		hour   => $hour,
+		minute => $minute,
+		second => 2 * $second,
+	);
+
+}
+sub to_dos {
+	my $tm = shift;
+
+	if (ref $tm ne 'Time::Moment') {
+		$tm = Time::Moment->from_string($tm);
+	}
+
+	($tm->year - 1980  << 25) +
+	($tm->month        << 21) +
+	($tm->day_of_month << 16) +
+	($tm->hour         << 11) +
+	($tm->minute       <<  5) +
+	($tm->second / 2);
+
+}
 
-=cut
 
 # Google Calendar time seems to count 32-day months from the day
 # before the Unix epoch. @noppers worked out how to do this.
@@ -136,11 +136,6 @@ sub to_google_calendar {
 	  +  $tm->second;
 }
 
-=head2 icq
-
-ICQ time is the number of days (with an allowed fractional part) since S<1899-12-30>.
-
-=cut
 
 #  ICQ time is the number of days since 1899-12-30, which is
 #  2,209,161,600 seconds before the Unix epoch. Days can have a
@@ -171,11 +166,6 @@ sub to_icq {
 	$t2->delta_nanoseconds($tm) / $NANOSECONDS_PER_DAY;
 }
 
-=head2 java
-
-Java time is the number of milliseconds since the Unix epoch.
-
-=cut
 
 # Java time is in milliseconds since the Unix epoch.
 sub java {
@@ -187,11 +177,6 @@ sub to_java {
 	_time2epoch($tm, 1000);
 }
 
-=head2 mozilla
-
-Mozilla time is the number of microseconds since the Unix epoch.
-
-=cut
 
 # Mozilla time is in microseconds since the Unix epoch.
 sub mozilla {
@@ -203,11 +188,6 @@ sub to_mozilla {
 	_time2epoch($tm, 1_000_000);
 }
 
-=head2 ole
-
-OLE time is the number of days since S<1899-12-30>, packed as a double-precision float in native format.
-
-=cut
 
 #  OLE time is the number of days since 1899-12-30, which is
 #  2,209,161,600 seconds before the Unix epoch.
@@ -230,11 +210,6 @@ sub to_ole {
 	return $epoch;
 }
 
-=head2 symbian
-
-Symbian time is the number of microseconds since the year 0.
-
-=cut
 
 # Symbian time is the number of microseconds since the year 0, which
 # is 62,167,219,200 seconds before the Unix epoch.
@@ -247,11 +222,6 @@ sub to_symbian {
 	_time2epoch($tm, 1_000_000, -62_167_219_200);
 }
 
-=head2 unix
-
-Unix time is the number of seconds since S<1970-01-01>.
-
-=cut
 
 # Unix time is the number of seconds since 1970-01-01.
 sub unix {
@@ -263,11 +233,6 @@ sub to_unix {
 	_time2epoch($tm);
 }
 
-=head2 uuid_v1
-
-UUID version 1 time (RFC 4122) is the number of hectonanoseconds S<(100 ns)> since S<1582-10-15>.
-
-=cut
 
 # UUID version 1 time (RFC 4122) is the number of hectonanoseconds
 # (100 ns) since 1582-10-15, which is 12,219,292,800 seconds before
@@ -281,11 +246,6 @@ sub to_uuid_v1 {
     _time2epoch($tm, 10_000_000, -12_219_292_800);
 }
 
-=head2 windows_date
-
-Windows date time (e.g., .NET) is the number of hectonanoseconds S<(100 ns)> since S<0001-01-01>.
-
-=cut
 
 # Windows date time (e.g., .NET) is the number of hectonanoseconds
 # (100 ns) since 0001-01-01, which is 62,135,596,800 seconds before
@@ -299,11 +259,6 @@ sub to_windows_date {
 	_time2epoch($tm, 10_000_000, -62_135_596_800);
 }
 
-=head2 windows_file
-
-Windows file time (e.g., NTFS) is the number of hectonanoseconds S<(100 ns)> since S<1601-01-01>.
-
-=cut
 
 # Windows file time (e.g., NTFS) is the number of hectonanoseconds
 # (100 ns) since 1601-01-01, which is 11,644,473,600 seconds before
@@ -347,6 +302,99 @@ sub _time2epoch {
 
 __END__
 
+=pod
+
+=for :stopwords Tim Heaney Ehlers Mary iopuckoi
+
+=head1 NAME
+
+Time::Moment::Epoch - Convert various epoch times to Time::Moment times.
+
+=head1 NAME
+
+Time::Moment::Epoch
+
+=head1 DESCRIPTION
+
+Convert various epoch times to and from datetimes using L<Time::Moment>.
+
+=head1 SYNOPSIS
+
+	use Time::Moment::Epoch;
+
+	say Time::Moment::Epoch::unix(1234567890);
+			# 2009-02-13T23:31:30Z
+	say Time::Moment::Epoch::to_unix('2009-02-13T23:31:30Z');
+			# 1234567890
+
+	say Time::Moment::Epoch::chrome(12879041490654321);
+			# 2009-02-13T23:31:30.654321Z
+	say Time::Moment::Epoch::to_chrome('2009-02-13T23:31:30.654321Z');
+			# 12879041490654321
+
+=head1 CONVERSIONS
+
+The following functions convert an epoch of the specified type to a Time::Moment object.
+
+They each have a corresponding C<to_$type> function which accepts a datetime string (in any format accepted by the C<from_string> method of L<Time::Moment>) and returns the corresponding epoch.
+
+=head2 apfs
+
+APFS time is the number of nanoseconds since the Unix epoch. Cf., APFS
+filesystem format (https://blog.cugu.eu/post/apfs/).
+
+=head2 chrome
+
+Chrome time is the number of microseconds since S<1601-01-01>.
+
+=head2 cocoa
+
+Cocoa time is the number of seconds since S<2001-01-01>.
+
+=head2 dos
+
+DOS time stores dates since S<1980-01-01> in bitfields.
+
+=head2 google_calendar
+
+Google Calendar time is 32-day months from the day before the Unix epoch.
+
+=head2 icq
+
+ICQ time is the number of days (with an allowed fractional part) since S<1899-12-30>.
+
+=head2 java
+
+Java time is the number of milliseconds since the Unix epoch.
+
+=head2 mozilla
+
+Mozilla time is the number of microseconds since the Unix epoch.
+
+=head2 ole
+
+OLE time is the number of days since S<1899-12-30>, packed as a double-precision float in native format.
+
+=head2 symbian
+
+Symbian time is the number of microseconds since the year 0.
+
+=head2 unix
+
+Unix time is the number of seconds since S<1970-01-01>.
+
+=head2 uuid_v1
+
+UUID version 1 time (RFC 4122) is the number of hectonanoseconds S<(100 ns)> since S<1582-10-15>.
+
+=head2 windows_date
+
+Windows date time (e.g., .NET) is the number of hectonanoseconds S<(100 ns)> since S<0001-01-01>.
+
+=head2 windows_file
+
+Windows file time (e.g., NTFS) is the number of hectonanoseconds S<(100 ns)> since S<1601-01-01>.
+
 =head1 SEE ALSO
 
 =over
@@ -355,4 +403,35 @@ __END__
 
 =back
 
+=head1 VERSION
+
+version 1.002
+
+=head1 AUTHOR
+
+Tim Heaney <heaney@cpan.org>
+
+=head1 CONTRIBUTORS
+
+=for stopwords Mary Ehlers iopuckoi
+
+=over 4
+
+=item *
+
+Mary Ehlers <regina.verb.ae@gmail.com>
+
+=item *
+
+iopuckoi <iopuckoi@gmail.com>
+
+=back
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2017 by Tim Heaney.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
